import * as effector from 'effector';
import { StoreWritable } from 'effector';

type NonEmptyTuple<T = unknown> = [T, ...T[]];
type AnyObject = Record<string, unknown>;
type ValueOf<T> = T[keyof T];
declare const CONTAINER_STATUS: {
    readonly idle: "idle";
    readonly pending: "pending";
    readonly done: "done";
    readonly fail: "fail";
    readonly off: "off";
};
type ContainerStatus = ValueOf<typeof CONTAINER_STATUS>;
type StartResult<T> = Promise<{
    api: T;
}> | {
    api: T;
};
type EnableResult = Promise<boolean> | boolean;
type AnyAPI = AnyObject | null;
type AnyStartFn = (...x: any) => StartResult<AnyAPI>;
type AnyContainer = {
    id: string;
    $status: StoreWritable<ContainerStatus>;
    start: AnyStartFn;
    dependsOn?: AnyContainer[];
    optionalDependsOn?: AnyContainer[];
    enable?: (..._: any) => EnableResult;
};
type AnyDeps = NonEmptyTuple<AnyContainer> | void;

declare const ERROR: {
    readonly CONTAINER_ID_EMPTY_STRING: "Container ID cannot be an empty string.";
    readonly depsIntersection: (intersection: string[], containerId: AnyContainer["id"]) => string;
};
type ContainerIdEmptyStringError = {
    id: never;
    error: typeof ERROR.CONTAINER_ID_EMPTY_STRING;
};

type ExtractDeps<D extends AnyContainer[]> = {
    [K in D[number] as Awaited<ReturnType<K['start']>>['api'] extends Record<string, never> ? never : K['id']]: Awaited<ReturnType<K['start']>>['api'];
};
type Params<Id extends string, API extends AnyAPI, Deps extends AnyDeps = void, OptionalDeps extends AnyDeps = void> = '' extends Id ? ContainerIdEmptyStringError : Deps extends void ? OptionalDeps extends void ? {
    id: Id;
    start: () => StartResult<API>;
    enable?: () => EnableResult;
} : {
    id: Id;
    optionalDependsOn: Exclude<OptionalDeps, void>;
    start: (_: void, optionalDeps: Partial<ExtractDeps<Exclude<OptionalDeps, void>>>) => StartResult<API>;
    enable?: (_: void, optionalDeps: Partial<ExtractDeps<Exclude<OptionalDeps, void>>>) => EnableResult;
} : OptionalDeps extends void ? {
    id: Id;
    dependsOn: Exclude<Deps, void>;
    start: (deps: ExtractDeps<Exclude<Deps, void>>) => StartResult<API>;
    enable?: (deps: ExtractDeps<Exclude<Deps, void>>) => EnableResult;
} : {
    id: Id;
    dependsOn: Exclude<Deps, void>;
    optionalDependsOn: Exclude<OptionalDeps, void>;
    start: (deps: ExtractDeps<Exclude<Deps, void>>, optionalDeps: Partial<ExtractDeps<Exclude<OptionalDeps, void>>>) => StartResult<API>;
    enable?: (deps: ExtractDeps<Exclude<Deps, void>>, optionalDeps: Partial<ExtractDeps<Exclude<OptionalDeps, void>>>) => EnableResult;
};
declare const createContainer: <Id extends string, API extends AnyAPI, Deps extends AnyDeps = void, OptionalDeps extends AnyDeps = void>(__params: Params<Id, API, Deps, OptionalDeps>) => Exclude<Params<Id, API, Deps, OptionalDeps>, ContainerIdEmptyStringError> & {
    $status: effector.StoreWritable<ContainerStatus>;
};

declare const compose: {
    up: (containers: AnyContainer[], config?: {
        debug?: boolean;
    }) => Promise<unknown>;
};

export { compose, createContainer };
